package com.kmall.admin.service.impl.mk;

import com.kmall.admin.dao.mk.MkDistDao;
import com.kmall.admin.entity.mk.MkDistEntity;
import com.kmall.admin.service.mk.MkDistService;
import com.kmall.admin.utils.ShiroUtils;
import com.kmall.common.utils.RRException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 分销商，distributorService实现类
 *
 * @author emato
 * @email admin@qhdswl.com
 * @date 2019-04-09 18:04:47
 */
@Service("mkDistService")
public class MkDistServiceImpl implements MkDistService {
    @Autowired
    private MkDistDao mkDistDao;

    @Override
    public MkDistEntity queryObject(Integer distId) {
        return mkDistDao.queryObject(distId);
    }

    @Override
    public List<MkDistEntity> queryList(Map<String, Object> map) {
        return mkDistDao.queryList(map);
    }

    @Override
    public List<MkDistEntity> queryAllByHier(Map<String, Object> map) {
        return mkDistDao.queryAllByHier(map);
    }


    @Override
    public MkDistEntity queryDistByGoodsId(Map<String, Object> map){
        return mkDistDao.queryDistByGoodsId(map);
    }
    @Override
    public int queryTotal(Map<String, Object> map) {
        return mkDistDao.queryTotal(map);
    }

    @Override
    public int save(MkDistEntity mkDist) {
        MkDistEntity mkDistEntity = mkDistDao.queryObjectByDistName(mkDist.getDistName());
        if (mkDistEntity != null) {
            throw new RRException("分销商名称已经存在");
        }
        MkDistEntity distEntity = mkDistDao.queryDistByStoreId(mkDist.getStoreId());
        if (distEntity != null) {
            throw new RRException("该门店已与其他分销商已创建关系，请重新选择门店");
        }

        mkDist.setCreaterSn(ShiroUtils.getUserEntity().getUsername());
        mkDist.setCreateTime(new Date());
        return mkDistDao.save(mkDist);
    }

    @Override
    public int update(MkDistEntity mkDist) {
        MkDistEntity mkDistEntity = mkDistDao.queryObjectByDistName(mkDist.getDistName());
        if (mkDistEntity != null && !mkDistEntity.getDistId().equals(mkDist.getDistId())) {
            throw new RRException("分销商名称已经存在");
        }
        MkDistEntity distEntity = mkDistDao.queryDistByStoreId(mkDist.getStoreId());
        if (distEntity != null && !distEntity.getDistId().equals(mkDist.getDistId())) {
            throw new RRException("该门店已与其他分销商已创建关系，请重新选择门店");
        }
        mkDist.setModerSn(ShiroUtils.getUserEntity().getUsername());
        mkDist.setModTime(new Date());
        return mkDistDao.update(mkDist);
    }

    @Override
    public int delete(Integer distId) {
        return mkDistDao.delete(distId);
    }

    @Override
    public int deleteBatch(Integer[]distIds) {
        return mkDistDao.deleteBatch(distIds);
    }
}
